#ifndef _CSMMM_H_
#define _CSMMM_H_  1

#include <initializer_list>
#include <iostream>
#include <vector>

// The number of pegs
static const int pegs = PEGS;
// The number of colors
static const int colors = COLORS;
// The maximal number of questions
static const int questions = QUESTIONS;
// Show found strategy
static const int show = SHOW;

typedef char Peg;

class Configuration {
  public:
    Configuration() {
    }

    explicit Configuration(Peg p) {
      for (int i = 0; i < pegs; ++i) {
        peg_tbl[i] = p;
      }
    }

    explicit Configuration(Peg const* p) {
      for (int i = 0; i < pegs; ++i) {
        peg_tbl[i] = p[i];
      }
    }

    Peg const & operator[](int i) const {
      return peg_tbl[i];
    }

    void swap(Configuration &c) {
      Peg tmp[pegs];
      memcpy(tmp, peg_tbl, pegs * sizeof(Peg));
      memcpy(peg_tbl, c.peg_tbl, pegs * sizeof(Peg));
      memcpy(c.peg_tbl, tmp, pegs * sizeof(Peg));
    }

  private:
    Peg peg_tbl[pegs];
};

static ::std::ostream & operator<<(::std::ostream& os, Configuration const& conf) {
  for (int i = 0; i < pegs; ++i)
    // os << (char)(conf[i] + 'a');
    os << (int)conf[i];
  return os;
}

class Maset {
  public:
    Maset() {
    }

    explicit Maset(size_t s) {
      m.reserve(s);
    }

    explicit Maset(::std::initializer_list<Configuration> list) {
      for (auto& element : list) {
        m.push_back(element);
      }
    }

    ~Maset() {
    }

    Configuration const& operator[](size_t i) const {
      return m[i];
    }

    size_t size() const {
      return m.size();
    }

    void push(Configuration const& conf) {
      m.push_back(conf);
    }

    void pop() {
      m.pop_back();
    }

    Configuration const& back() const {
      return m.back();
    }

    void swap(Maset& mas) {
      m.swap(mas.m);
    }

  protected:
    ::std::vector<Configuration> m;
};

static ::std::ostream& operator<<(::std::ostream& os, Maset const& maset) {
  for (size_t i = 0; i < maset.size(); ++i)
    os << maset[i] << '\n';
  return os;
}

#endif
